@@ -28,12 +28,14 @@ ONE_DEGREE = 1000. * 10000.8 / 90.
 
 EARTH_RADIUS = 6371 * 1000
 
+
 def to_rad(x):
     return x / 180. * mod_math.pi
 
+
 def haversine_distance(latitude_1, longitude_1, latitude_2, longitude_2):
     """
-    Haversine distance between two points.
+    Haversine distance between two points, expressed in meters.
 
     Implemented from http://www.movable-type.co.uk/scripts/latlong.html
     """
@@ -49,6 +51,7 @@ def haversine_distance(latitude_1, longitude_1, latitude_2, longitude_2):
 
     return d
 
+
 def length(locations=None, _3d=None):
     locations = locations or []
     if not locations:
@@ -69,23 +72,26 @@ def length(locations=None, _3d=None):
                 length += d
     return length
 
+
 def length_2d(locations=None):
-    """ 2-dimensional length of locations (only latitude and longitude, no elevation """
+    """ 2-dimensional length (meters) of locations (only latitude and longitude, no elevation). """
     locations = locations or []
     return length(locations, False)
 
+
 def length_3d(locations=None):
-    """ 3-dimensional length of locations (is uses latitude, longitude and elevation). """
+    """ 3-dimensional length (meters) of locations (it uses latitude, longitude, and elevation). """
     locations = locations or []
     return length(locations, True)
 
+
 def calculate_max_speed(speeds_and_distances):
     """
-    Compute average distance and standard deviation for distance. Extreemes 
-    in dinstances are usually extreemes in speeds, so we will ignore them, 
+    Compute average distance and standard deviation for distance. Extremes
+    in distances are usually extremes in speeds, so we will ignore them,
     here.
 
-    speeds_and_distances must be a list containing pairs of (speed, distance) 
+    speeds_and_distances must be a list containing pairs of (speed, distance)
     for every point in a track segment.
     """
     assert speeds_and_distances
@@ -109,24 +115,26 @@ def calculate_max_speed(speeds_and_distances):
 
     # sort by speed:
     speeds = list(map(lambda speed_and_distance: speed_and_distance[0], filtered_speeds_and_distances))
-    if not isinstance(speeds, list): # python3
+    if not isinstance(speeds, list):  # python3
         speeds = list(speeds)
     if not speeds:
         return None
     speeds.sort()
 
-    # Even here there may be some extreemes => ignore the last 5%:
+    # Even here there may be some extremes => ignore the last 5%:
     index = int(len(speeds) * 0.95)
     if index >= len(speeds):
         index = -1
 
     return speeds[index]
 
+
 def calculate_uphill_downhill(elevations):
     if not elevations:
         return 0, 0
 
     size = len(elevations)
+
     def __filter(n):
         current_ele = elevations[n]
         if current_ele is None:
@@ -134,7 +142,7 @@ def calculate_uphill_downhill(elevations):
         if 0 < n < size - 1:
             previous_ele = elevations[n-1]
             next_ele = elevations[n+1]
-            if previous_ele is not None and current_ele is not None and next_ele is not None: 
+            if previous_ele is not None and current_ele is not None and next_ele is not None:
                 return previous_ele*.3 + current_ele*.4 + next_ele*.3
         return current_ele
 
@@ -152,16 +160,17 @@ def calculate_uphill_downhill(elevations):
 
     return uphill, downhill
 
+
 def distance(latitude_1, longitude_1, elevation_1, latitude_2, longitude_2, elevation_2,
              haversine=None):
     """
     Distance between two points. If elevation is None compute a 2d distance
 
-    if haversine==True -- haversine will be used for every computations, 
+    if haversine==True -- haversine will be used for every computations,
     otherwise...
 
-    Haversine distance will be used for distant points where elevation makes a 
-    small difference, so it is ignored. That's because haversine is 5-6 times 
+    Haversine distance will be used for distant points where elevation makes a
+    small difference, so it is ignored. That's because haversine is 5-6 times
     slower than the dummy distance algorithm (which is OK for most GPS tracks).
     """
 
@@ -180,6 +189,7 @@ def distance(latitude_1, longitude_1, elevation_1, latitude_2, longitude_2, elev
 
     return mod_math.sqrt(distance_2d ** 2 + (elevation_1 - elevation_2) ** 2)
 
+
 def elevation_angle(location1, location2, radians=False):
     """ Uphill/downhill angle between two locations. """
     if location1.elevation is None or location2.elevation is None:
@@ -198,6 +208,7 @@ def elevation_angle(location1, location2, radians=False):
 
     return 180 * angle / mod_math.pi
 
+
 def distance_from_line(point, line_point_1, line_point_2):
     """ Distance of point from a line given with two points. """
     assert point, point
@@ -207,7 +218,7 @@ def distance_from_line(point, line_point_1, line_point_2):
     a = line_point_1.distance_2d(line_point_2)
 
     if a == 0:
-        raise Exception('Invalid line (%s, %s)' % (line_point_1, line_point_2))
+        return line_point_1.distance_2d(point)
 
     b = line_point_1.distance_2d(point)
     c = line_point_2.distance_2d(point)
@@ -216,6 +227,7 @@ def distance_from_line(point, line_point_1, line_point_2):
 
     return 2. * mod_math.sqrt(abs(s * (s - a) * (s - b) * (s - c))) / a
 
+
 def get_line_equation_coefficients(location1, location2):
     """
     Get line equation coefficients for:
@@ -231,6 +243,7 @@ def get_line_equation_coefficients(location1, location2):
         b = location1.latitude - location1.longitude * a
         return float(1), float(-a), float(-b)
 
+
 def simplify_polyline(points, max_distance):
     """Does Ramer-Douglas-Peucker algorithm for simplification of polyline """
 
@@ -241,10 +254,10 @@ def simplify_polyline(points, max_distance):
 
     # Use a "normal" line just to detect the most distant point (not its real distance)
     # this is because this is faster to compute than calling distance_from_line() for
-    # every point. 
+    # every point.
     #
-    # This is an approximation and may have some errors near the poles and if 
-    # the points are too distant, but it should be good enough for most use 
+    # This is an approximation and may have some errors near the poles and if
+    # the points are too distant, but it should be good enough for most use
     # cases...
     a, b, c = get_line_equation_coefficients(begin, end)
 
@@ -263,9 +276,10 @@ def simplify_polyline(points, max_distance):
     if real_max_distance < max_distance:
         return [begin, end]
 
-    return (simplify_polyline(points[:tmp_max_distance_position + 2], max_distance) + 
+    return (simplify_polyline(points[:tmp_max_distance_position + 2], max_distance) +
             simplify_polyline(points[tmp_max_distance_position + 1:], max_distance)[1:])
 
+
 class Location:
     """ Generic geographical location """
 
@@ -299,12 +313,72 @@ class Location:
     def elevation_angle(self, location, radians=False):
         return elevation_angle(self, location, radians)
 
-    def move(self, latitude_diff, longitude_diff):
-        self.latitude += latitude_diff
-        self.longitude += longitude_diff
-		
+    def move(self, location_delta):
+        self.latitude, self.longitude = location_delta.move(self)
+
+    def __add__(self, location_delta):
+        latitude, longitude = location_delta.move(self)
+        return Location(latitude, longitude)
+
     def __str__(self):
         return '[loc:%s,%s@%s]' % (self.latitude, self.longitude, self.elevation)
 
+    def __repr__(self):
+        if self.elevation is None:
+            return 'Location(%s, %s)' % (self.latitude, self.longitude)
+        else:
+            return 'Location(%s, %s, %s)' % (self.latitude, self.longitude, self.elevation)
+
     def __hash__(self):
-        return mod_utils.hash_object(self, 'latitude', 'longitude', 'elevation') 
+        return mod_utils.hash_object(self, 'latitude', 'longitude', 'elevation')
+
+
+class LocationDelta:
+    """
+    Intended to use similar to timestamp.timedelta, but for Locations.
+    """
+
+    NORTH = 0
+    EAST = 90
+    SOUTH = 180
+    WEST = 270
+
+    def __init__(self, distance=None, angle=None, latitude_diff=None, longitude_diff=None):
+        """
+        Version 1:
+            Distance (in meters).
+            angle_from_north *clockwise*. 
+            ...must be given
+        Version 2:
+            latitude_diff and longitude_diff
+            ...must be given
+        """
+        if (distance is not None) and (angle is not None):
+            if (latitude_diff is not None) or (longitude_diff is not None):
+                raise Exception('No lat/lon diff if using distance and angle!')
+            self.distance = distance
+            self.angle_from_north = angle
+            self.move_function = self.move_by_angle_and_distance
+        elif (latitude_diff is not None) and (longitude_diff is not None):
+            if (distance is not None) or (angle is not None):
+                raise Exception('No distance/angle if using lat/lon diff!')
+            this.latitude_diff  = latitude_diff
+            this.longitude_diff = longitude_diff
+            self.move_function = self.move_by_lat_lon_diff
+
+    def move(self, location):
+        """
+        Move location by this timedelta.
+        """
+        return self.move_function(location)
+
+    def move_by_angle_and_distance(self, location):
+        coef = mod_math.cos(location.latitude / 180. * mod_math.pi)
+        vertical_distance_diff   = mod_math.sin((90 - self.angle_from_north) / 180. * mod_math.pi) / ONE_DEGREE
+        horizontal_distance_diff = mod_math.cos((90 - self.angle_from_north) / 180. * mod_math.pi) / ONE_DEGREE
+        lat_diff = self.distance * vertical_distance_diff
+        lon_diff = self.distance * horizontal_distance_diff / coef
+        return location.latitude + lat_diff, location.longitude + lon_diff
+
+    def move_by_lat_lon_diff(self, location):
+        return location.latitude  + self.latitude_diff, location.longitude + self.longitude_diff
