@@ -27,11 +27,12 @@ try:
     import lxml.etree as mod_etree
 except:
     mod_etree = None
-    pass # LXML not available
+    pass  # LXML not available
 
 from . import gpx as mod_gpx
 from . import utils as mod_utils
 
+
 class XMLParser:
     """
     Used when lxml is not available. Uses standard minidom.
@@ -85,16 +86,17 @@ class XMLParser:
             return node.attributes[attribute].nodeValue
         return None
 
+
 class LXMLParser:
     """
-    Used when lxml is available. 
+    Used when lxml is available.
     """
 
     def __init__(self, xml):
         assert mod_etree
 
         if mod_utils.PYTHON_VERSION[0] == '3':
-            # In python 3 all strings are unicode and for some reason lxml 
+            # In python 3 all strings are unicode and for some reason lxml
             # don't like unicode strings with XMLs declared as UTF-8:
             self.xml = xml.encode('utf-8')
         else:
@@ -143,23 +145,23 @@ class LXMLParser:
     def get_node_attribute(self, node, attribute):
         return node.attrib.get(attribute)
 
+
 def parse_time(string):
     if not string:
         return None
-    try:
-        return mod_datetime.datetime.strptime(string, mod_gpx.DATE_FORMAT)
-    except ValueError as e:
-        if mod_re.match(r'^.*\.\d+Z$', string):
-            string = mod_re.sub(r'\.\d+Z', 'Z', string)
+    if 'T' in string:
+        string = string.replace('T', ' ')
+    if 'Z' in string:
+        string = string.replace('Z', '')
+    for date_format in mod_gpx.DATE_FORMATS:
         try:
-            return mod_datetime.datetime.strptime(string, mod_gpx.DATE_FORMAT)
+            return mod_datetime.datetime.strptime(string, date_format)
         except ValueError as e:
-            mod_logging.error('Invalid timestamp %s' % string)
-            return None
+            pass
+    return None
 
 
 class GPXParser:
-
     def __init__(self, xml_or_file=None, parser=None):
         """
         Parser may be lxml of minidom. If you set to None then lxml will be used if installed
@@ -182,8 +184,8 @@ class GPXParser:
         """
         Parses the XML file and returns a GPX object.
 
-        It will throw GPXXMLSyntaxException if the XML file is invalid or 
-        GPXException if the XML file is valid but something is wrong with the 
+        It will throw GPXXMLSyntaxException if the XML file is invalid or
+        GPXException if the XML file is valid but something is wrong with the
         GPX data.
         """
         try:
@@ -207,22 +209,21 @@ class GPXParser:
             mod_logging.debug('Error in:\n%s\n-----------\n' % self.xml)
             mod_logging.exception(e)
 
-            # The library should work in the same way regardless of the 
-            # underlying XML parser that's why the exception thrown 
-            # here is GPXXMLSyntaxException (instead of simply throwing the 
-            # original minidom or lxml exception e). 
+            # The library should work in the same way regardless of the
+            # underlying XML parser that's why the exception thrown
+            # here is GPXXMLSyntaxException (instead of simply throwing the
+            # original minidom or lxml exception e).
             #
-            # But, if the user need the original exception (lxml or minidom) 
+            # But, if the user need the original exception (lxml or minidom)
             # it is available with GPXXMLSyntaxException.original_exception:
             raise mod_gpx.GPXXMLSyntaxException('Error parsing XML: %s' % str(e), e)
 
     def __parse_dom(self):
-
         node = self.xml_parser.get_first_child(name='gpx')
         if node is None:
             raise mod_gpx.GPXException('Document must have a `gpx` root node.')
         if self.xml_parser.get_node_attribute(node, "creator"):
-          self.gpx.creator = self.xml_parser.get_node_attribute(node, "creator")
+            self.gpx.creator = self.xml_parser.get_node_attribute(node, "creator")
 
         for node in self.xml_parser.get_children(node):
             node_name = self.xml_parser.get_node_name(node)
@@ -287,7 +288,8 @@ class GPXParser:
         lon = mod_utils.to_number(lon)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node),
+                                        default=None, nan_value=None)
 
         time_node = self.xml_parser.get_first_child(node, 'time')
         time_str = self.xml_parser.get_node_data(time_node)
@@ -318,9 +320,9 @@ class GPXParser:
         pdop = mod_utils.to_number(self.xml_parser.get_node_data(pdop_node))
 
         return mod_gpx.GPXWaypoint(latitude=lat, longitude=lon, elevation=elevation,
-            time=time, name=name, description=desc, symbol=sym,
-            type=type, comment=comment, horizontal_dilution=hdop,
-            vertical_dilution=vdop, position_dilution=pdop)
+                                   time=time, name=name, description=desc, symbol=sym,
+                                   type=type, comment=comment, horizontal_dilution=hdop,
+                                   vertical_dilution=vdop, position_dilution=pdop)
 
     def _parse_route(self, node):
         name_node = self.xml_parser.get_first_child(node, 'name')
@@ -355,7 +357,8 @@ class GPXParser:
         lon = mod_utils.to_number(lon)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node),
+                                        default=None, nan_value=None)
 
         time_node = self.xml_parser.get_first_child(node, 'time')
         time_str = self.xml_parser.get_node_data(time_node)
@@ -386,12 +389,15 @@ class GPXParser:
         pdop = mod_utils.to_number(self.xml_parser.get_node_data(pdop_node))
 
         return mod_gpx.GPXRoutePoint(lat, lon, elevation, time, name, desc, sym, type, comment,
-            horizontal_dilution = hdop, vertical_dilution = vdop, position_dilution = pdop)
+                                     horizontal_dilution=hdop, vertical_dilution=vdop, position_dilution=pdop)
 
     def __parse_track(self, node):
         name_node = self.xml_parser.get_first_child(node, 'name')
         name = self.xml_parser.get_node_data(name_node)
 
+        type_node = self.xml_parser.get_first_child(node, 'type')
+        type = self.xml_parser.get_node_data(type_node)
+
         description_node = self.xml_parser.get_first_child(node, 'desc')
         description = self.xml_parser.get_node_data(description_node)
 
@@ -399,6 +405,7 @@ class GPXParser:
         number = mod_utils.to_number(self.xml_parser.get_node_data(number_node))
 
         track = mod_gpx.GPXTrack(name, description, number)
+        track.type = type
 
         child_nodes = self.xml_parser.get_children(node)
         for child_node in child_nodes:
@@ -434,7 +441,8 @@ class GPXParser:
         time = parse_time(time_str)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node),
+                                        default=None, nan_value=None)
 
         sym_node = self.xml_parser.get_first_child(node, 'sym')
         symbol = self.xml_parser.get_node_data(sym_node)
@@ -458,9 +466,8 @@ class GPXParser:
         speed = mod_utils.to_number(self.xml_parser.get_node_data(speed_node))
 
         return mod_gpx.GPXTrackPoint(latitude=latitude, longitude=longitude, elevation=elevation, time=time,
-            symbol=symbol, comment=comment, horizontal_dilution=hdop, vertical_dilution=vdop,
-            position_dilution=pdop, speed=speed, name=name)
-
+                                     symbol=symbol, comment=comment, horizontal_dilution=hdop, vertical_dilution=vdop,
+                                     position_dilution=pdop, speed=speed, name=name)
 
 
 if __name__ == '__main__':
